package Test::Repocop::ACLWrapper;
use strict;
use warnings;
use Test::Repocop::ExternalACL;
use Test::Repocop::Options;

our $aclfile;

eval {
    require 'ALTLinux/ACL.pm';
    require 'Test/Repocop/ALTLinuxACL.pm';
    push @Test::Repocop::Options::LONGOPT, @ALTLinux::ACL::LONGOPT;
};

my $no_altlinux_acl=$@;
my $use_altlinux_acl;

push @Test::Repocop::Options::LONGOPT, (
	"acl-file=s" => \$aclfile,
	"acl-altlinux" => \$use_altlinux_acl,
    );

sub new {
    my $class=shift;
    return Test::Repocop::ExternalACL->new($aclfile) if $aclfile;
    if ($use_altlinux_acl) {
	if ($no_altlinux_acl) {
	    warn "Loading ALT ACL failed: $no_altlinux_acl";
	} else {
	    return Test::Repocop::ALTLinuxACL->new(ALTLinux::ACL->new);
	} 
    }
    warn "Test::Repocop::ACLWrapper: no ACL provider selected." if $Test::Repocop::Options::verbose;
    return;
}

1;

=head1	NAME

Test::Repocop::ACLWrapper - repocop library to provide package ACL.

=head1	DESCRIPTION

TODO

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop. 

=head1	COPYING

Copyright (c) 2008-2012 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

