package Test::Repocop::Common;
use Test::Repocop::Options;
use File::Basename;
use File::Path qw(make_path remove_tree);

use strict;

sub rpmfile2key {
    $_=shift;
    s!^.*/!!;
    s/\.rpm$//;
    return $_;
}

sub key_is_source {
    return $_[0]=~/\.src$/;
}

my $repocop_test_tmpdirname='tmp';

sub set_test_environment {
    my ($testname)=@_;
    $ENV{'REPOCOP_STATEDIR'}=$Test::Repocop::Options::repocop_statedir;
    $ENV{'REPOCOP_TEST_STATEDIR'}="$Test::Repocop::Options::repocop_statedir/$testname";
    $ENV{'REPOCOP_TEST_TMPDIR'}="$repocop_cachedir/$repocop_test_tmpdirname/$testname";
    $ENV{'REPOCOP_TEST_DBDIR'}=$repocop_test_dbdir;
    $ENV{'REPOCOP_TEST_NAME'}=$testname;
    my $repocop_test_db="$repocop_test_dbdir/$testname.db";
    $ENV{'REPOCOP_TEST_DB'}=$repocop_test_db if -e $repocop_test_db;
    
    $ENV{'REPOCOP_DISTROTEST_DBDIR'}=$repocop_test_dbdir unless $ENV{'REPOCOP_DISTROTEST_DBDIR'};
}

sub mkdir_test_environment {
    my ($testname)=@_;
    &set_test_environment($testname);
    make_path($ENV{'REPOCOP_TEST_STATEDIR'}, $ENV{'REPOCOP_TEST_TMPDIR'});
}

sub cleanup_test_environment {
    my ($testname)=@_;
    remove_tree("$repocop_cachedir/$repocop_test_tmpdirname/$testname");
    # only empty directories are deleted
    rmdir "$Test::Repocop::Options::repocop_statedir/$testname";
}

sub cleanup_test_environments {
    remove_tree("$repocop_cachedir/$repocop_test_tmpdirname");
    foreach my $testname (@_) {
	# only empty directories are deleted
	rmdir "$Test::Repocop::Options::repocop_statedir/$testname";
    }
}

sub discard_test_environment {
    my ($testname)=@_;
    remove_tree("$repocop_cachedir/$repocop_test_tmpdirname/$testname");
    remove_tree("$Test::Repocop::Options::repocop_statedir/$testname");
}

sub glob_tests {
    my ($file,@dirs)=@_;
    my @retval;
    foreach (@dirs) {
	push @retval, glob "$_/*/$file";
    }
    return @retval;
}

sub filepath2testname {
    return basename(dirname($_[0]));
}

#sub is_srpm {
#    return $_[0]=~/\.src$/;
#}

sub find_test_dir {
    my ($testname)=@_;
    my @tests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/{pkgtests,srctests,pkgcollectors,srccollectors}");
    foreach my $dir(@tests_dir) {
	$dir .= '/'.$testname;
	return $dir if -d $dir;
    }
}

# public static members
sub specfile2srcid {
    my $id=basename(shift());
    #$id=~s!^.*/!!;
    $id=~s/\.spec$//;
    $id.='.src' if $id!~/\.src$/;
    return $id;
}

# public static members
sub patchname2srcid {
    my $id=basename(shift());
    $id=~s/(?:-(?:digest|cummulative|safe))?\.(?:patch|diff)$//;
    $id.='.src' if $id!~/\.src$/;
    return $id;
}


1;

=head1	NAME

Test::Repocop::Common - common private subroutines for repocop.

=head1	DESCRIPTION

Common private subroutines for repocop. 
The interface is subject to change without notice.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop. 

=head1	COPYING

Copyright (c) 2008 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

