package Test::Repocop::FixscriptFactory;

use strict;
use warnings;
use Cwd;
use Carp;
use Test::Repocop::Common;
use File::Basename;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;
our $VERSION=0.002;

# MOVE to upper script ?
#$RPM::Source::Transformation::DiffWritePlayer::opt_diff_by_srpm_dir='.';
#$RPM::Source::Transformation::DiffWritePlayer::opt_diff_author_name='Repocop Q. A. Robot';
#$RPM::Source::Transformation::DiffWritePlayer::opt_diff_author_email='repocop@altlinux.org';
#$RPM::Source::Transformation::DiffWritePlayer::opt_diff_email_subject='repocop NMU';

our $verbose=1;

# our -- hack ?
our $alttoolsdir=dirname($0);
$alttoolsdir=cwd() if $alttoolsdir !~ m'^/home/';

# for cross-references in testnames
my @fixscriptdirs = ('/usr/share/repocop/fixscripts');
push @fixscriptdirs, $alttoolsdir.'/fixscripts' if -d $alttoolsdir.'/fixscripts';
push @INC, @fixscriptdirs;

our %fixscriptmap;
our @SPECHOOKS;

my @fixscripts= map {s!^/usr/share/repocop/fixscripts/!!; $_} glob('/usr/share/repocop/fixscripts/*.pl');
push @fixscripts, map {s!^$alttoolsdir/fixscripts/'!!; $_} glob ($alttoolsdir.'/fixscripts/*.pl') if -d $alttoolsdir.'/fixscripts';
my @postcleanscripts;
foreach my $dir (@fixscriptdirs) {
    push @postcleanscripts, map {basename($_)} map {s!.pl$!!;$_} glob ("$dir/postclean-*.pl");
}
our %test_with_fixscript=map {$_=>1} map {s!.pl$!!;$_} @fixscripts,@postcleanscripts;
print STDERR "Repocop::FixscriptFactory: available fixscripts:", join(',',keys(%test_with_fixscript)),"\n" if $verbose>2;

sub register_messages {
    my (undef,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-repocop_testnames=> [],
	-repocop_pkgnames=> [],
    })
}

sub transformations {
    my (undef,$opt,$resourceobj)=@_;
    my @transformations = (
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_POST_BEGIN,
	    -name => 'initialize default changelog',
	    -opt_handler_sub=>\&opt_handler_add_default_changelog_AT_BEGIN)
	);

    my $testnames=$opt->{-repocop_testnames};
    for (my $i=0;$i<@$testnames;$i++) {
	push @transformations, RPM::Source::Transformation::RepocopFixWrapper->new(
	    -testname => $testnames->[$i],
	    -pkgname => $opt->{-repocop_pkgnames}->[$i],
	    -drop_me_test_modification => undef,
	    ) if $test_with_fixscript{$testnames->[$i]};
    }
    # postclean scripts
    foreach my $name (@postcleanscripts) {
	push @transformations, RPM::Source::Transformation::RepocopFixWrapper->new(
	    -testname => $name,
	    -pkgname => '',
	    -drop_me_test_modification => 1,
	    );
    }
    return @transformations;
}

sub opt_handler_add_default_changelog_AT_BEGIN {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    $spec->add_default_changelog('- applied repocop fixes:'."\n");
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub __load_pl {
    my $testname=shift;
    return unless $test_with_fixscript{$testname};
    print STDERR "Repocop::FixscriptFactory: __load_pl: proceed with $testname:\n" if $verbose>3;
    unless ($fixscriptmap{$testname}) {
	eval {
	    require $testname.'.pl';
	};
	warn "can't load fixscript for $testname: skipped\nmsg:$@\n" if $@ and $verbose;
	if (@SPECHOOKS) {
	    $fixscriptmap{$testname}=$SPECHOOKS[0];
	    $#{SPECHOOKS}=-1;
	} else {
	    print STDERR "ERROR: Repocop::FixscriptFactory: fixscript for $testname does not have a hook!\n";
	}
    } else {
	print STDERR "Repocop::FixscriptFactory: __load_pl: fixscriptmap exists:$fixscriptmap{$testname}\n" if $verbose>3;
    }
    return $fixscriptmap{$testname};
}

package RPM::Source::Transformation::RepocopFixWrapper;

use strict;
use warnings;

our $verbose=0;

sub new {
    my $class = shift;
    my $self = {
	-priority => 0,
	@_
    };
    die "new: -testname is required" unless defined $self->{-testname};
    die "new: -pkgname is required" unless defined $self->{-pkgname};
    
    $self->{-subject}='repocop fix for '.$self->{-testname};
    $self->{-name}=$self->{-testname};
    # TODO !!! load description!
    #$self->{-description}=$description{$self->{-testname}};
    bless $self, $class;
    return $self;
}

sub apply {
    my ($self,$spec,$parent,$opt,$resourceobj)=@_;
    die "$spec is undefined" unless $spec;
    &_call_fixscript($spec,$parent,$self->{-testname},$self->{-pkgname},$self->{-drop_me_test_modification});
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub _call_fixscript {
    my ($spec,$parent,$testname,$pkg, $test_modification)=@_;
    my $msg_pkg=$pkg;
    $msg_pkg||='the spec file';
    print STDERR "trying $testname for $msg_pkg\n" if $verbose>1;
    my $fixscriptref=&Test::Repocop::FixscriptFactory::__load_pl($testname);
    if ($fixscriptref) {
	print STDERR "applying $testname for $msg_pkg\n" if $verbose;
	&Test::Repocop::Common::set_test_environment($testname);
	my $before=$spec->get_spec() if $test_modification;
#	$spec->{WARN_NOTAPPLIED}++;
	&{$fixscriptref}($spec,$pkg,$pkg);
#	$spec->{WARN_NOTAPPLIED}--;

	return if $test_modification and $spec->get_spec() eq $before;
	$spec->add_default_changelog("  * $testname for $msg_pkg\n");
	print STDERR "applied $testname for $msg_pkg\n" if $verbose;
    } else {
	print STDERR "no fixscript for $testname for $msg_pkg\n" if $verbose;
    }
}

1;
