package RPM::Source::Transformation::Resource::Repocop;

use strict;
use warnings;
use Carp;

use Data::Array2ArrayMap::Hash::XSTree;

use Test::Repocop::Common;
use Test::Repocop::Options;
use Test::Repocop::TestDB;
use Test::Repocop::Metadata;

use RPM::Source::Transformation::Resource;
our @ISA=qw/RPM::Source::Transformation::Resource/;
push @RPM::Source::TransformContainer::RESOURCE_CLASSES, __PACKAGE__;

# no need; rule is not to publish experimental patchgenerators
#$Repocop::arg::reportlevel='experimental';

our $verbose=0;
my $RPM2TEST=Data::Array2ArrayMap::Hash::XSTree->new();
my $RPM2NAME=Data::Array2ArrayMap::Hash::XSTree->new();
my ($metadata,$sourceid2specfilename);
my %NAME2SOURCEID;

*LONGOPT=*Test::Repocop::Options::LONGOPT;
*longopt_usage=*Test::Repocop::Options::longopt_usage;

sub process_options {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;

    &Test::Repocop::Options::process_options();
    &Test::Repocop::Options::die_if_nothing_to_report();

    $metadata=Test::Repocop::Metadata->new();
    $sourceid2specfilename=Test::Repocop::Metadata::sourceid2specfilename();
    my $iterator3=Test::Repocop::TestDB->new()->get_pkg_test_status_iterator();

    # hack to process all known test results
    undef %Repocop::arg::pkgtable;
    while (my ($rpm,$test,$status)=$iterator3->iterate3_filtered()) {
	#print STDERR "$rpm:$test:$status\n";
	my $sourceid=$metadata->sourceid($rpm);
	my $name=$metadata->name($rpm);
	$RPM2TEST->append([$sourceid],[$test]);
	$RPM2NAME->append([$sourceid],[$name]);
	$NAME2SOURCEID{$name}=$sourceid if &Test::Repocop::Common::key_is_source($rpm);
    }
}


sub prepare_resource {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $name=$spec->main_section->get_tag('Name');
    my $sourceid=$NAME2SOURCEID{$name};
    die "can't find sourceid for $name\n" unless defined $sourceid;
    my @tests=$RPM2TEST->get([$sourceid]);
    my @tests_with_fixscript=grep {$Test::Repocop::FixscriptFactory::test_with_fixscript{$_}} @tests;
    if (scalar @tests_with_fixscript) {
	my @names=$RPM2NAME->get([$sourceid]);
	$opt->{-repocop_testnames}=\@names;
	$opt->{-repocop_pkgnames}=\@tests;

	# diff options
	my $diffname=$sourceid;
	$diffname=~s/\.src$//;
	$opt->{-diff_name}=$diffname;
	$opt->{-diff_specname}=$sourceid2specfilename->{$sourceid};
	# end diff options
    }
}

package RPM::Source::Shared::ResourceContainer;

# nothing...
 
1;

__END__
