package Test::Repocop::TestDB::Backend::Filesystem;

use strict;
use warnings;

use File::Path qw(make_path remove_tree);
use File::Basename;
use Test::Repocop::Options; # just for cachedir && verbose :(

our $repocop_test_outputdirname='testdb';
our $repocop_test_backend='fsprefix';
our $repocop_test_backend_prefix_width=1;

sub new {
    my $class=shift;
    my $self = {
    };
    bless $self, $class;
    &__detect_backend();
    return $self;
}

### internal hack
sub __detect_backend {
    if (-e "$repocop_cachedir/testdb-fs-md5-3") {
	$repocop_test_outputdirname='testdb-fs-md5-3';
	$repocop_test_backend='fsmd5prefix';
	$repocop_test_backend_prefix_width=3;
    } else {
	$repocop_test_outputdirname='testdb';
	$repocop_test_backend='fsprefix';
	$repocop_test_backend_prefix_width=1;
    }
}

sub setup_environment {
    my $self=shift;
    my $repocop_test_outputdir="$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname";
    make_path($repocop_test_outputdir);
    # for repocop-test-* utils
    $ENV{'REPOCOP_TEST_OUTPUTDIR'}=$repocop_test_outputdir;
    $ENV{'REPOCOP_BACKEND_PREFIX_WIDTH'}=$repocop_test_backend_prefix_width;
    # set PATH
    $ENV{'PATH'}='/usr/libexec/repocop/backends/'.$repocop_test_backend.':'.$ENV{'PATH'};
}

sub remove_test {
    my $self=shift;
    return unless @_;
    my @findclause;
    foreach my $testname (@_) {
	push @findclause, '-name',$testname,'-or','-name',$testname.'+++*', '-or';
    }
    pop @findclause if @findclause;
    system ('find',"$repocop_cachedir/$repocop_test_outputdirname",'-type','f','(',@findclause,')','-delete') == 0 || die "error: $!";
}

# static internal
sub __get_touchfile {
    my ($repocop_pkg_key,$testname)=@_;
    my $prefix=$repocop_pkg_key;
    if ($repocop_test_backend eq 'fsmd5prefix') {
	$prefix=`echo $repocop_pkg_key|md5sum`;
    }
    $prefix=substr($prefix,0,$repocop_test_backend_prefix_width);
    my $path="$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname/$prefix/$repocop_pkg_key";
    $path.="/$testname" if defined $testname;
    return $path;
}

sub is_test_performed {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    return -e &__get_touchfile($repocop_pkg_key,$testname);
}

sub get_test_mtime {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    return -M &__get_touchfile($repocop_pkg_key,$testname);
}

sub set_test_mtime {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    my $repocop_test_result=&__get_touchfile($repocop_pkg_key,$testname);
    make_path(dirname($repocop_test_result));
    system('touch',$repocop_test_result)==0 or die "touch $repocop_test_result: $!";
}

sub clear_test_result_and_mtime {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    my $repocop_test_result=&__get_touchfile($repocop_pkg_key,$testname);
    unlink <$repocop_test_result*>;
}

sub delete_pkg {
    my $self=shift;
    my ($repocop_pkg_key)=@_;
    return unless $repocop_pkg_key;
    remove_tree(&__get_touchfile($repocop_pkg_key));
}

sub get_pkg_test_status_iterator {
    return Test::Repocop::TestDB::Backend::Filesystem::TestIterator->new();
}

sub get_pkg_test_status_result_iterator {
    return Test::Repocop::TestDB::Backend::Filesystem::TestIterator->new();
}

sub purge {
    my $self=shift;
    my $testPurger=Test::Repocop::TestDB::Backend::Filesystem::__PackageIterator->new();
    while ($_=$testPurger->iterate_pkg()) {
	next if &Repocop::arg::skip_pkg($_);
	print "deleting: $_\n" if $verbose;
	$self->delete_pkg($_);
    }
}

package Test::Repocop::TestDB::Backend::Filesystem::TestIterator;

use strict;
use warnings;

use Test::Repocop::Options; # just for cachedir, metadata && verbose :(
use Test::Repocop::Metadata;
use File::Path qw(make_path remove_tree);
use File::Basename;

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
	CUR_TEST_FILENAME => '',
	FILTER_NICK=> 0,
    };
    bless $self, $class;
    open ($self->{FD}, " find $repocop_cachedir/$Test::Repocop::TestDB::Backend::Filesystem::repocop_test_outputdirname/ -type f -name '*+++*' |");
    if (scalar %Repocop::arg::EXCLUDEPACKAGER or scalar %Repocop::arg::INCLUDEPACKAGER) {
	$self->{METADATA}=Test::Repocop::Metadata->new();
	$self->{FILTER_NICK}=1;
    }
    return $self;
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate3 {
    my $self=shift;
    $_=readline $self->{FD};
    return unless $_;
    chomp $_;
    $self->{CUR_TEST_FILENAME}=$_;
    s!^$repocop_cachedir/$repocop_test_outputdirname/[^/]+/!!o;
    #print "$_\n";
    my ($rpmkey,$teststatus)=split '/';
    my ($test,$status)=split '\+\+\+', $teststatus;
    return ($rpmkey,$test,$status);
}

sub iterate3_filtered {
    my $self=shift;
    my ($rpm,$test,$status);
    do {
	($rpm,$test,$status)=$self->iterate3();
    } while $rpm && (&Repocop::arg::skip_test($test) || &Repocop::arg::skip_pkg($rpm) || &Repocop::arg::skip_status($status) || ($self->{FILTER_NICK} and &Repocop::arg::skip_nick($self->{METADATA}->nick($rpm))) );
    return unless $rpm;
    return ($rpm,$test,$status);
}


sub iterate4_filtered {
    my $self=shift;
    my ($rpm,$test,$status)=$self->iterate3_filtered();
    return unless $rpm;
    my $result=$self->__result();
    return ($rpm,$test,$status,$result);
}

sub __result {
    my $self=shift;
    return &_read_file($self->{CUR_TEST_FILENAME});
}


sub _read_file {
    my $filename=shift; 
    return "" unless -s $filename;
    open my $fh, $filename or die $!;
    local $/; # enable localized slurp mode
    my $content = <$fh>;
    close $fh;
    return $content;
}

package Test::Repocop::TestDB::Backend::Filesystem::__PackageIterator;

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
    };
    bless $self, $class;
    open ($self->{FD}, "find $Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname/".('?'x$repocop_test_backend_prefix_width)." -type d |");
    return $self;
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate_pkg {
    my $self=shift;
    while (1) {
	$_=readline $self->{FD};
	return unless $_;
	chomp $_;
	#my $filename=$_;
	s!^$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname/[^/]+/?!!o;
	next unless $_;
	return $_;
    }
}

1;

